@@ -508,7 +508,7 @@ whitespace:
 
 				case ']':
 
-					if (top->type == json_array) {
+					if (top && top->type == json_array) {
 						flags = (flags & ~(flag_need_comma | flag_seek_value)) | flag_next;
 					} else {              sprintf(error, "%d:%d: Unexpected ]", cur_line, e_off);
 						              goto e_failed; }
