@@ -185,8 +185,16 @@ char *nick_gen(bee_user_t *bu)
 		   accents don't just get stripped. Note that it depends on
 		   LC_CTYPE being set to something other than C/POSIX. */
 		if (!(irc && irc->status & IRC_UTF8_NICKS)) {
-			part = asc = g_convert_with_fallback(part, -1, "ASCII//TRANSLIT",
-			                                     "UTF-8", "", NULL, NULL, NULL);
+			asc = g_convert_with_fallback(part, -1, "ASCII//TRANSLIT", "UTF-8", "", NULL, NULL, NULL);
+
+			if (!asc) {
+				/* If above failed, try again without //TRANSLIT.
+				   //TRANSLIT is a GNU iconv special and is not POSIX.
+				   Other platforms may not support it. */
+				asc = g_convert_with_fallback(part, -1, "ASCII", "UTF-8", "", NULL, NULL, NULL);
+			}
+
+			part = asc;
 		}
 
 		if (part && chop && (s = strchr(part, chop))) {
@@ -213,6 +221,22 @@ char *nick_gen(bee_user_t *bu)
 	return NULL;
 }
 
+/* Used for nicks and channel names too! */
+void underscore_dedupe(char nick[MAX_NICK_LENGTH + 1])
+{
+	if (strlen(nick) < (MAX_NICK_LENGTH - 1)) {
+		nick[strlen(nick) + 1] = 0;
+		nick[strlen(nick)] = '_';
+	} else {
+		/* We've got no more space for underscores,
+		   so truncate it and replace the last three
+		   chars with a random "_XX" suffix */
+		int len = truncate_utf8(nick, MAX_NICK_LENGTH - 3);
+		nick[len] = '_';
+		g_snprintf(nick + len + 1, 3, "%2x", rand());
+	}
+}
+
 void nick_dedupe(bee_user_t *bu, char nick[MAX_NICK_LENGTH + 1])
 {
 	irc_t *irc = (irc_t *) bu->bee->ui_data;
@@ -223,17 +247,8 @@ void nick_dedupe(bee_user_t *bu, char nick[MAX_NICK_LENGTH + 1])
 	   subtle changes to make it unique. */
 	while (!nick_ok(irc, nick) ||
 	       ((iu = irc_user_by_name(irc, nick)) && iu->bu != bu)) {
-		if (strlen(nick) < (MAX_NICK_LENGTH - 1)) {
-			nick[strlen(nick) + 1] = 0;
-			nick[strlen(nick)] = '_';
-		} else {
-			/* We've got no more space for underscores,
-			   so truncate it and replace the last three
-			   chars with a random "_XX" suffix */
-			int len = truncate_utf8(nick, MAX_NICK_LENGTH - 3);
-			nick[len] = '_';
-			g_snprintf(nick + len + 1, 3, "%2x", rand());
-		}
+
+		underscore_dedupe(nick);
 
 		if (inf_protection-- == 0) {
 			g_snprintf(nick, MAX_NICK_LENGTH + 1, "xx%x", rand());
@@ -360,9 +375,13 @@ int nick_lc(irc_t *irc, char *nick)
 	int i;
 
 	if (tab['A'] == 0) {
+		/* initialize table so nonchars are mapped to themselves */
+		for (i = 0; i < sizeof(tab); i++) {
+			tab[i] = i;
+		}
+		/* replace uppercase chars with lowercase chars */
 		for (i = 0; nick_lc_chars[i]; i++) {
 			tab[(int) nick_uc_chars[i]] = nick_lc_chars[i];
-			tab[(int) nick_lc_chars[i]] = nick_lc_chars[i];
 		}
 	}
 
