@@ -186,7 +186,7 @@ void otr_update_uflags(ConnContext *context, irc_user_t *u);
 
 /* update op/voice flag of given user according to encryption state and settings
    returns 0 if neither op_buddies nor voice_buddies is set to "encrypted",
-   i.e. msgstate should be announced seperately */
+   i.e. msgstate should be announced separately */
 int otr_update_modeflags(irc_t *irc, irc_user_t *u);
 
 /* show general info about the OTR subsystem; called by 'otr info' */
@@ -215,9 +215,16 @@ gboolean otr_disconnect_user(irc_t *irc, irc_user_t *u);
 /* close all active OTR connections */
 void otr_disconnect_all(irc_t *irc);
 
+/* modifies string in-place, replacing \x03 with '?',
+   as a quick way to prevent remote users from messing with irc colors */
+static char *otr_filter_colors(char *msg);
+
 /* functions to be called for certain events */
 static const struct irc_plugin otr_plugin;
 
+#define OTR_COLOR_TRUSTED "03"     /* green */
+#define OTR_COLOR_UNTRUSTED "05"   /* red */
+
 /*** routines declared in otr.h: ***/
 
 #ifdef OTR_BI
@@ -433,7 +440,8 @@ char *otr_filter_msg_in(irc_user_t *iu, char *msg, int flags)
 	struct im_connection *ic = iu->bu->ic;
 
 	/* don't do OTR on certain (not classic IM) protocols, e.g. twitter */
-	if (ic->acc->prpl->options & OPT_NOOTR) {
+	if (ic->acc->prpl->options & OPT_NOOTR ||
+	    iu->bu->flags & BEE_USER_NOOTR) {
 		return msg;
 	}
 
@@ -450,7 +458,7 @@ char *otr_filter_msg_in(irc_user_t *iu, char *msg, int flags)
 		return NULL;
 	} else if (!newmsg) {
 		/* this was a non-OTR message */
-		return msg;
+		return otr_filter_colors(msg);
 	} else {
 		/* we're done with the original msg, which will be caller-freed. */
 		return newmsg;
@@ -471,7 +479,8 @@ char *otr_filter_msg_out(irc_user_t *iu, char *msg, int flags)
 	 */
 
 	/* don't do OTR on certain (not classic IM) protocols, e.g. twitter */
-	if (ic->acc->prpl->options & OPT_NOOTR) {
+	if (ic->acc->prpl->options & OPT_NOOTR ||
+	    iu->bu->flags & BEE_USER_NOOTR) {
 		return msg;
 	}
 
@@ -741,6 +750,51 @@ void op_create_instag(void *opdata, const char *account, const char *protocol)
 	}
 }
 
+static char *otr_filter_colors(char *msg)
+{
+	return str_reject_chars(msg, "\x02\x03", '?');
+}
+
+/* returns newly allocated string */
+static char *otr_color_encrypted(char *msg, char *color, gboolean is_query) {
+	char **lines;
+	GString *out;
+	int i;
+
+	lines = g_strsplit(msg, "\n", -1);
+
+	/* up to 4 extra chars per line (e.g., '\x03' + ("03"|"05") + ' ') */
+	out = g_string_sized_new(strlen(msg) + g_strv_length(lines) * 4);
+	
+	for (i = 0; lines[i]; i++) {
+		char *line = lines[i];
+
+		if (i != 0) {
+			g_string_append_c(out, '\n');
+
+		} else if (is_query && g_strncasecmp(line, "/me ", 4) == 0) {
+			/* in a query window, keep "/me " uncolored at the beginning */
+			line += 4;
+			g_string_append(out, "/me ");
+		}
+
+		g_string_append_c(out, '\x03');
+		g_string_append(out, color);
+
+		/* comma in first place could mess with the color code */
+		if (line[0] == ',') {
+			/* insert a space between color spec and message */
+			g_string_append_c(out, ' ');
+		}
+
+		g_string_append(out, otr_filter_colors(line));
+	}
+
+	g_strfreev(lines);
+
+	return g_string_free(out, FALSE);
+}
+
 void op_convert_msg(void *opdata, ConnContext *ctx, OtrlConvertType typ,
                     char **dst, const char *src)
 {
@@ -751,44 +805,28 @@ void op_convert_msg(void *opdata, ConnContext *ctx, OtrlConvertType typ,
 
 	if (typ == OTRL_CONVERT_RECEIVING) {
 		char *msg = g_strdup(src);
-		char *buf = msg;
 
 		/* HTML decoding */
 		if (set_getbool(&ic->bee->set, "otr_does_html") &&
 		    !(ic->flags & OPT_DOES_HTML) &&
 		    set_getbool(&ic->bee->set, "strip_html")) {
 			strip_html(msg);
+
+			/* msg is borrowed by *dst (unless the next if decides to color it) */
 			*dst = msg;
 		}
 
 		/* coloring */
 		if (set_getbool(&ic->bee->set, "otr_color_encrypted")) {
-			int color;                /* color according to f'print trust */
-			char *pre = "", *sep = "";    /* optional parts */
 			const char *trust = ctx->active_fingerprint->trust;
+			char *color = (trust && *trust) ? OTR_COLOR_TRUSTED : OTR_COLOR_UNTRUSTED;
+			gboolean is_query = (irc_user_msgdest(iu) == irc->user->nick);
 
-			if (trust && trust[0] != '\0') {
-				color = 3;   /* green */
-			} else {
-				color = 5;   /* red */
+			/* the return value of otr_color_encrypted() is borrowed by *dst */
+			*dst = otr_color_encrypted(msg, color, is_query);
 
-			}
-			/* in a query window, keep "/me " uncolored at the beginning */
-			if (g_strncasecmp(msg, "/me ", 4) == 0
-			    && irc_user_msgdest(iu) == irc->user->nick) {
-				msg += 4;  /* skip */
-				pre = "/me ";
-			}
-
-			/* comma in first place could mess with the color code */
-			if (msg[0] == ',') {
-				/* insert a space between color spec and message */
-				sep = " ";
-			}
-
-			*dst = g_strdup_printf("%s\x03%.2d%s%s\x0F", pre,
-			                       color, sep, msg);
-			g_free(buf);
+			/* this branch doesn't need msg */
+			g_free(msg);
 		}
 	} else {
 		/* HTML encoding */
@@ -1356,8 +1394,8 @@ void display_otr_message(void *opdata, ConnContext *ctx, const char *fmt, ...)
 	va_end(va);
 
 	if (u) {
-		/* display as a notice from this particular user */
-		irc_usernotice(u, "%s", msg);
+		/* just show this as a regular message */
+		irc_usermsg(u, "<<\002OTR\002>> %s", msg);
 	} else {
 		irc_rootmsg(irc, "[otr] %s", msg);
 	}
@@ -1694,6 +1732,9 @@ OtrlPrivKey *match_privkey(irc_t *irc, const char **args)
 	}
 	*p = '\0';
 
+	/* remove trailing whitespace */
+	g_strchomp(prefix);
+
 	/* find first key which matches the given prefix */
 	n = strlen(prefix);
 	for (k = irc->otr->us->privkey_root; k; k = k->next) {
