@@ -49,8 +49,6 @@ typedef enum {
 
 	JFLAG_GTALK =  0x100000,        /* Is Google Talk, as confirmed by iq discovery */
 	JFLAG_HIPCHAT = 0x200000,       /* Is hipchat, because prpl->name says so */
-
-	JFLAG_SASL_FB = 0x10000,        /* Trying Facebook authentication. */
 } jabber_flags_t;
 
 typedef enum {
@@ -112,6 +110,8 @@ struct jabber_data {
 	GSList *filetransfers;
 	GSList *streamhosts;
 	int have_streamhosts;
+
+	char *muc_host;
 };
 
 struct jabber_away_state {
@@ -227,6 +227,9 @@ struct jabber_transfer {
 #define XMLNS_DELAY        "urn:xmpp:delay"                                      /* XEP-0203 */
 #define XMLNS_XDATA        "jabber:x:data"                                       /* XEP-0004 */
 #define XMLNS_GMAILNOTIFY  "google:mail:notify"                                  /* Not a XEP */
+#define XMLNS_CARBONS      "urn:xmpp:carbons:2"                                  /* XEP-0280 */
+#define XMLNS_FORWARDING   "urn:xmpp:forward:0"                                  /* XEP-0297 */
+#define XMLNS_HINTS        "urn:xmpp:hints"                                      /* XEP-0334 */
 #define XMLNS_CHATSTATES   "http://jabber.org/protocol/chatstates"               /* XEP-0085 */
 #define XMLNS_DISCO_INFO   "http://jabber.org/protocol/disco#info"               /* XEP-0030 */
 #define XMLNS_DISCO_ITEMS  "http://jabber.org/protocol/disco#items"              /* XEP-0030 */
@@ -337,7 +340,6 @@ int sasl_oauth2_get_refresh_token(struct im_connection *ic, const char *msg);
 int sasl_oauth2_refresh(struct im_connection *ic, const char *refresh_token);
 
 extern const struct oauth2_service oauth2_service_google;
-extern const struct oauth2_service oauth2_service_facebook;
 
 /* conference.c */
 struct groupchat *jabber_chat_join(struct im_connection *ic, const char *room, const char *nick, const char *password);
@@ -355,5 +357,7 @@ void jabber_chat_invite(struct groupchat *c, char *who, char *message);
 int jabber_get_hipchat_profile(struct im_connection *ic);
 xt_status jabber_parse_hipchat_profile(struct im_connection *ic, struct xt_node *node, struct xt_node *orig);
 xt_status hipchat_handle_success(struct im_connection *ic, struct xt_node *node);
+char *hipchat_make_channel_slug(const char *name);
+char *hipchat_guess_channel_name(struct im_connection *ic, const char *name);
 
 #endif
