@@ -72,12 +72,12 @@ const struct msn_status_code msn_status_code_list[] =
 	{ 205, "Invalid (non-existent) handle",                         0 },
 	{ 206, "Domain name missing",                                   0 },
 	{ 207, "Already logged in",                                     0 },
-	{ 208, "Invalid handle",                                        STATUS_SB_IM_SPARE },
+	{ 208, "Invalid handle",                                        0 },
 	{ 209, "Forbidden nickname",                                    0 },
 	{ 210, "Buddy list too long",                                   0 },
 	{ 215, "Handle is already in list",                             0 },
-	{ 216, "Handle is not in list",                                 STATUS_SB_IM_SPARE },
-	{ 217, "Person is off-line or non-existent",                    STATUS_SB_IM_SPARE },
+	{ 216, "Handle is not in list",                                 0 },
+	{ 217, "Person is off-line or non-existent",                    0 },
 	{ 218, "Already in that mode",                                  0 },
 	{ 219, "Handle is already in opposite list",                    0 },
 	{ 223, "Too many groups",                                       0 },
@@ -112,7 +112,7 @@ const struct msn_status_code msn_status_code_list[] =
 	{ 710, "Invalid CVR parameters",                                STATUS_FATAL },
 	{ 711, "Write is blocking",                                     STATUS_FATAL },
 	{ 712, "Session is overloaded",                                 STATUS_FATAL },
-	{ 713, "Calling too rapidly",                                   STATUS_SB_IM_SPARE },
+	{ 713, "Calling too rapidly",                                   0 },
 	{ 714, "Too many sessions",                                     STATUS_FATAL },
 	{ 715, "Not expected/Invalid argument/action",                  0 },
 	{ 717, "Bad friend file",                                       STATUS_FATAL },
