@@ -21,6 +21,11 @@
 
 #include "bitlbee.h"
 
+#ifdef WITH_PURPLE
+/* For compatibility with builds that include both purple and this oscar module */
+#include "aim_prefixes.h"
+#endif
+
 /* XXX adjust these based on autoconf-detected platform */
 typedef guint32 aim_snacid_t;
 typedef guint16 flap_seqnum_t;
@@ -870,7 +875,7 @@ int aim_sncmp(const char *a, const char *b);
 /*
  * SNAC Family: Internal Messages
  *
- * This isn't truely a SNAC family either, but using
+ * This isn't truly a SNAC family either, but using
  * these, we can integrated non-SNAC services into
  * the SNAC-centered libfaim callback structure.
  *
