@@ -468,16 +468,11 @@ int twitter_url_len_diff(gchar *msg, unsigned int target_len)
 
 	g_regex_match(regex, msg, 0, &match_info);
 	while (g_match_info_matches(match_info)) {
-		gchar *s, *url;
+		gchar *url;
 
 		url = g_match_info_fetch(match_info, 2);
 		url_len_diff += target_len - g_utf8_strlen(url, -1);
 
-		/* Add another character for https://t.co/... URLs */
-		if ((s = g_match_info_fetch(match_info, 3))) {
-			url_len_diff += 1;
-			g_free(s);
-		}
 		g_free(url);
 		g_match_info_next(match_info, NULL);
 	}
@@ -540,7 +535,7 @@ static void twitter_init(account_t * acc)
 
 	if (strcmp(acc->prpl->name, "twitter") == 0) {
 		def_url = TWITTER_API_URL;
-		def_tul = "22";
+		def_tul = "23";
 		def_mentions = "true";
 	} else {                /* if( strcmp( acc->prpl->name, "identica" ) == 0 ) */
 		def_url = IDENTICA_API_URL;
@@ -677,14 +672,14 @@ static void twitter_logout(struct im_connection *ic)
 	// Set the status to logged out.
 	ic->flags &= ~OPT_LOGGED_IN;
 
-	// Remove the main_loop function from the function queue.
-	b_event_remove(td->main_loop_id);
-
-	if (td->timeline_gc) {
-		imcb_chat_free(td->timeline_gc);
-	}
-
 	if (td) {
+		// Remove the main_loop function from the function queue.
+		b_event_remove(td->main_loop_id);
+
+		if (td->timeline_gc) {
+			imcb_chat_free(td->timeline_gc);
+		}
+
 		if (td->filter_update_id > 0) {
 			b_event_remove(td->filter_update_id);
 		}
