@@ -162,6 +162,12 @@ static void cmd_identify(irc_t *irc, char **cmd)
 		irc->status |= USTATUS_IDENTIFIED;
 		irc_umode_set(irc, "+R", 1);
 
+		if (irc->caps & CAP_SASL) {
+			irc_user_t *iu = irc->user;
+			irc_send_num(irc, 900, "%s!%s@%s %s :You are now logged in as %s",
+				iu->nick, iu->user, iu->host, iu->nick, iu->nick);
+		}
+
 		bitlbee_whatsnew(irc);
 
 		/* The following code is a bit hairy now. With takeover
@@ -238,6 +244,12 @@ static void cmd_register(irc_t *irc, char **cmd)
 		irc->status |= USTATUS_IDENTIFIED;
 		irc_umode_set(irc, "+R", 1);
 
+		if (irc->caps & CAP_SASL) {
+			irc_user_t *iu = irc->user;
+			irc_send_num(irc, 900, "%s!%s@%s %s :You are now logged in as %s",
+				iu->nick, iu->user, iu->host, iu->nick, iu->nick);
+		}
+
 		/* Set this var now, or anyone who logs in to his/her
 		   newly created account for the first time gets the
 		   whatsnew story. */
@@ -412,7 +424,11 @@ static void cmd_account(irc_t *irc, char **cmd)
 		prpl = find_protocol(cmd[2]);
 
 		if (prpl == NULL) {
-			irc_rootmsg(irc, "Unknown protocol");
+			if (is_protocol_disabled(cmd[2])) {
+				irc_rootmsg(irc, "Protocol disabled in global config");
+			} else {
+				irc_rootmsg(irc, "Unknown protocol");
+			}
 			return;
 		}
 
@@ -1221,6 +1237,7 @@ static void cmd_group(irc_t *irc, char **cmd)
 static void cmd_transfer(irc_t *irc, char **cmd)
 {
 	GSList *files = irc->file_transfers;
+	GSList *next;
 
 	enum { LIST, REJECT, CANCEL };
 	int subcmd = LIST;
@@ -1238,7 +1255,8 @@ static void cmd_transfer(irc_t *irc, char **cmd)
 		subcmd = CANCEL;
 	}
 
-	for (; files; files = g_slist_next(files)) {
+	for (; files; files = next) {
+		next = files->next;
 		file_transfer_t *file = files->data;
 
 		switch (subcmd) {
