@@ -1,4 +1,4 @@
-  /********************************************************************\
+/********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
   * Copyright 2002-2004 Wilmer van der Gaast and others                *
@@ -21,8 +21,8 @@
 
   You should have received a copy of the GNU General Public License with
   the Debian GNU/Linux distribution in /usr/share/common-licenses/GPL;
-  if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-  Suite 330, Boston, MA  02111-1307  USA
+  if not, write to the Free Software Foundation, Inc., 51 Franklin St.,
+  Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #define BITLBEE_CORE
@@ -45,15 +45,18 @@ static storage_t *storage_init_single(const char *name)
 
 	for (gl = storage_backends; gl; gl = gl->next) {
 		st = gl->data;
-		if (strcmp(st->name, name) == 0)
+		if (strcmp(st->name, name) == 0) {
 			break;
+		}
 	}
 
-	if (gl == NULL) 
+	if (gl == NULL) {
 		return NULL;
+	}
 
-	if (st->init)
+	if (st->init) {
 		st->init();
+	}
 
 	return st;
 }
@@ -63,29 +66,31 @@ GList *storage_init(const char *primary, char **migrate)
 	GList *ret = NULL;
 	int i;
 	storage_t *storage;
-	
+
 	register_storage_backend(&storage_xml);
-	
+
 	storage = storage_init_single(primary);
-	if (storage == NULL && storage->save == NULL)
+	if (storage == NULL && storage->save == NULL) {
 		return NULL;
+	}
 
 	ret = g_list_append(ret, storage);
 
 	for (i = 0; migrate && migrate[i]; i++) {
 		storage = storage_init_single(migrate[i]);
-	
-		if (storage)
+
+		if (storage) {
 			ret = g_list_append(ret, storage);
+		}
 	}
 
 	return ret;
 }
 
-storage_status_t storage_check_pass (const char *nick, const char *password)
+storage_status_t storage_check_pass(const char *nick, const char *password)
 {
 	GList *gl;
-	
+
 	/* Loop until we don't get NO_SUCH_USER */
 
 	for (gl = global.storage; gl; gl = gl->next) {
@@ -93,85 +98,88 @@ storage_status_t storage_check_pass (const char *nick, const char *password)
 		storage_status_t status;
 
 		status = st->check_pass(nick, password);
-		if (status != STORAGE_NO_SUCH_USER)
+		if (status != STORAGE_NO_SUCH_USER) {
 			return status;
+		}
 	}
-	
+
 	return STORAGE_NO_SUCH_USER;
 }
 
-storage_status_t storage_load (irc_t * irc, const char *password)
+storage_status_t storage_load(irc_t * irc, const char *password)
 {
 	GList *gl;
-	
-	if (irc && irc->status & USTATUS_IDENTIFIED)
+
+	if (irc && irc->status & USTATUS_IDENTIFIED) {
 		return STORAGE_OTHER_ERROR;
-	
+	}
+
 	/* Loop until we don't get NO_SUCH_USER */
 	for (gl = global.storage; gl; gl = gl->next) {
 		storage_t *st = gl->data;
 		storage_status_t status;
 
 		status = st->load(irc, password);
-		if (status == STORAGE_OK)
-		{
+		if (status == STORAGE_OK) {
 			GSList *l;
-			for( l = irc_plugins; l; l = l->next )
-			{
+			for (l = irc_plugins; l; l = l->next) {
 				irc_plugin_t *p = l->data;
-				if( p->storage_load )
-					p->storage_load( irc );
+				if (p->storage_load) {
+					p->storage_load(irc);
+				}
 			}
 			return status;
 		}
-		
-		if (status != STORAGE_NO_SUCH_USER) 
+
+		if (status != STORAGE_NO_SUCH_USER) {
 			return status;
+		}
 	}
-	
+
 	return STORAGE_NO_SUCH_USER;
 }
 
-storage_status_t storage_save (irc_t *irc, char *password, int overwrite)
+storage_status_t storage_save(irc_t *irc, char *password, int overwrite)
 {
 	storage_status_t st;
 	GSList *l;
-	
+
 	if (password != NULL) {
 		/* Should only use this in the "register" command. */
-		if (irc->password || overwrite)
+		if (irc->password || overwrite) {
 			return STORAGE_OTHER_ERROR;
-		
+		}
+
 		irc_setpass(irc, password);
 	} else if ((irc->status & USTATUS_IDENTIFIED) == 0) {
 		return STORAGE_NO_SUCH_USER;
 	}
-	
-	st = ((storage_t *)global.storage->data)->save(irc, overwrite);
-	
-	for( l = irc_plugins; l; l = l->next )
-	{
+
+	st = ((storage_t *) global.storage->data)->save(irc, overwrite);
+
+	for (l = irc_plugins; l; l = l->next) {
 		irc_plugin_t *p = l->data;
-		if( p->storage_save )
-			p->storage_save( irc );
+		if (p->storage_save) {
+			p->storage_save(irc);
+		}
 	}
-	
+
 	if (password != NULL) {
 		irc_setpass(irc, NULL);
 	}
-	
+
 	return st;
 }
 
-storage_status_t storage_remove (const char *nick, const char *password)
+storage_status_t storage_remove(const char *nick, const char *password)
 {
 	GList *gl;
 	storage_status_t ret = STORAGE_OK;
 	gboolean ok = FALSE;
 	GSList *l;
-	
-	/* Remove this account from all storage backends. If this isn't 
-	 * done, the account will still be usable, it'd just be 
+
+	/* Remove this account from all storage backends. If this isn't
+	 * done, the account will still be usable, it'd just be
 	 * loaded from a different backend. */
 	for (gl = global.storage; gl; gl = gl->next) {
 		storage_t *st = gl->data;
@@ -179,61 +187,20 @@ storage_status_t storage_remove (const char *nick, const char *password)
 
 		status = st->remove(nick, password);
 		ok |= status == STORAGE_OK;
-		if (status != STORAGE_NO_SUCH_USER && status != STORAGE_OK)
+		if (status != STORAGE_NO_SUCH_USER && status != STORAGE_OK) {
 			ret = status;
-	}
-	
-	/* If at least one succeeded, remove plugin data. */
-	if( ok )
-		for( l = irc_plugins; l; l = l->next )
-		{
-			irc_plugin_t *p = l->data;
-			if( p->storage_remove )
-				p->storage_remove( nick );
 		}
-	
+	}
+
+	/* If at least one succeeded, remove plugin data. */
+	if (ok) {
+		for (l = irc_plugins; l; l = l->next) {
+			irc_plugin_t *p = l->data;
+			if (p->storage_remove) {
+				p->storage_remove(nick);
+			}
+		}
+	}
+
 	return ret;
 }
-
-#if 0
-Not using this yet. Test thoroughly before adding UI hooks to this function.
-
-storage_status_t storage_rename (const char *onick, const char *nnick, const char *password)
-{
-	storage_status_t status;
-	GList *gl = global.storage;
-	storage_t *primary_storage = gl->data;
-	irc_t *irc;
-
-	/* First, try to rename in the current write backend, assuming onick 
-	 * is stored there */
-	status = primary_storage->rename(onick, nnick, password);
-	if (status != STORAGE_NO_SUCH_USER)
-		return status;
-
-	/* Try to load from a migration backend and save to the current backend. 
-	 * Explicitly remove the account from the migration backend as otherwise 
-	 * it'd still be usable under the old name */
-	
-	irc = g_new0(irc_t, 1);
-	status = storage_load(onick, password, irc);
-	if (status != STORAGE_OK) {
-		irc_free(irc);
-		return status;
-	}
-
-	g_free(irc->nick);
-	irc->nick = g_strdup(nnick);
-
-	status = storage_save(irc, FALSE);
-	if (status != STORAGE_OK) {
-		irc_free(irc);
-		return status;
-	}
-	irc_free(irc);
-
-	storage_remove(onick, password);
-
-	return STORAGE_OK;
-}
-#endif
